@@ -68,7 +68,8 @@ var gClearButton = null;
 var gFilterBox = null;
 var gSearchTimer = null;
 var gSearchInput = null;
-
+var gShowAllKeysElement = null;
+var gTreeChildren = null;
 
 function enigmailKeyManagerLoad() {
   DEBUG_LOG("enigmailKeyManager.js: enigmailKeyManagerLoad\n");
@@ -76,25 +77,38 @@ function enigmailKeyManagerLoad() {
   gFilterBox = document.getElementById("filterKey");
   gClearButton = document.getElementById("clearFilter");
   gSearchInput = document.getElementById("filterKey");
-  gUserList.addEventListener('click', enigmailOnClick, true);
+  gShowAllKeysElement = document.getElementById("showAllKeys");
+  gTreeChildren = document.getElementById("pgpKeyListChildren");
+  if (EnigGetPref("keyManShowAllKeys")) {
+    gShowAllKeysElement.setAttribute("checked", "true");
+  }
 
+  gUserList.addEventListener('click', enigmailOnClick, true);
+  document.getElementById("bcEnableKey").setAttribute("label", EnigGetString("keyMan.disableKey"))
+  
   window.enigIpcRequest = null;
 
+  document.getElementById("pleaseWait").showPopup(gSearchInput, -1, -1, "tooltip", "after_end", "");
   document.getElementById("statusText").value = EnigGetString("keyMan.loadingKeys");
   document.getElementById("progressBar").removeAttribute("collapsed");
   window.setTimeout(loadkeyList, 100);
+  gSearchInput.focus();
 }
 
+function displayFullList() {
+  return (gShowAllKeysElement.getAttribute("checked") == "true");
+}
 
 function loadkeyList() {
   DEBUG_LOG("enigmailKeyManager.js: loadkeyList\n");
 
   enigmailBuildList(false);
+  showOrHideAllKeys();
+  document.getElementById("pleaseWait").hidePopup();
   document.getElementById("statusText").value=" ";
   document.getElementById("progressBar").setAttribute("collapsed", "true");
 }
 
-
 function enigmailRefreshKeys() {
   DEBUG_LOG("enigmailKeyManager.js: enigmailRefreshKeys\n");
   var keyList = enigmailGetSelectedKeys();
@@ -102,7 +116,7 @@ function enigmailRefreshKeys() {
   for (var i=0; i<keyList.length; i++) {
     gEnigLastSelectedKeys[keyList[i]] = 1;
   }
-  var treeChildren = document.getElementById("pgpKeyListChildren");
+  var treeChildren = gTreeChildren;
   while (treeChildren.firstChild) {
     treeChildren.removeChild(treeChildren.firstChild);
   }
@@ -111,7 +125,7 @@ function enigmailRefreshKeys() {
 }
 
 
-function enigmailBuildList(refresh) {
+function old_enigmail_Build_List_unused(refresh) {
   DEBUG_LOG("enigmailKeyManager.js: enigmailBuildList\n");
   var keyListObj = {};
 
@@ -120,9 +134,9 @@ function enigmailBuildList(refresh) {
   gKeyList = keyListObj.keyList;
   gKeySortList = keyListObj.keySortList;
 
-  gUserList.currentItem=null;
+  gUserList.currentItem = null;
 
-  var treeChildren=document.getElementById("pgpKeyListChildren");
+  var treeChildren = gTreeChildren;
 
   var selectedItems=[];
   for (var i=0; i < gKeySortList.length; i++) {
@@ -145,6 +159,84 @@ function enigmailBuildList(refresh) {
       treeChildren.appendChild(treeItem);
   }
 
+  if (selectedItems.length>0) {
+    gUserList.view.selection.select(selectedItems[0]);
+    for (i=1; i<selectedItems.length; i++) {
+      gUserList.view.selection.rangedSelect(selectedItems[i], selectedItems[i], true)
+    }
+  }
+}
+
+
+function enigmailBuildList(refresh) {
+  DEBUG_LOG("enigmailKeyManager.js: enigmailBuildList\n");
+
+  var keyListObj = {};
+
+  EnigLoadKeyList(refresh, keyListObj);
+
+  gKeyList = keyListObj.keyList;
+  gKeySortList = keyListObj.keySortList;
+
+  gUserList.currentItem = null;
+
+  var treeChildren = gTreeChildren;
+
+  var selectedItems=[];
+  for (var i=0; i < gKeySortList.length; i++) {
+    var keyId = gKeySortList[i].keyId;
+    if (gEnigLastSelectedKeys && typeof(gEnigLastSelectedKeys[keyId]) != "undefined")
+      selectedItems.push(i);
+    var treeItem=null;
+    treeItem=enigUserSelCreateRow(gKeyList[keyId], -1)
+    treeItem.setAttribute("container", "true");
+    var subChildren = document.createElement("treechildren");
+    treeItem.appendChild(subChildren);
+    var uidItem = document.createElement("treeitem");
+    subChildren.appendChild(uidItem);
+    var uidRow=document.createElement("treerow");
+    var uidCell=document.createElement("treecell");
+    uidCell.setAttribute("label", EnigGetString("keylist.noOtherUids"));
+    uidRow.appendChild(uidCell);
+    uidItem.appendChild(uidRow);
+    uidItem.setAttribute("keytype", "none");
+    uidItem.setAttribute("id", keyId);
+
+    var uidChildren = document.createElement("treechildren");
+    uidItem.appendChild(uidChildren);
+    var uatItem = document.createElement("treeitem");
+    uatItem.setAttribute("id", keyId);
+    uatItem.setAttribute("keytype", "none");
+
+    subChildren.appendChild(uatItem);
+    var uatRow=document.createElement("treerow");
+    var uatCell=document.createElement("treecell");
+    uatCell.setAttribute("label", EnigGetString("keylist.noPhotos"));
+    uatRow.appendChild(uatCell);
+    uatItem.appendChild(uatRow);
+    var uatChildren = document.createElement("treechildren");
+    uatItem.appendChild(uatChildren);
+
+    for (var subkey=0; subkey<gKeyList[keyId].SubUserIds.length; subkey++) {
+      var subItem=enigUserSelCreateRow(gKeyList[keyId], subkey);
+      if (gKeyList[keyId].SubUserIds[subkey].type == "uat") {
+        uatItem.setAttribute("container", "true");
+        uatCell.setAttribute("label", EnigGetString("keylist.hasPhotos"));
+        uatChildren.appendChild(subItem);
+        uatItem.setAttribute("open", "true");
+      }
+      else {
+        uidItem.setAttribute("container", "true");
+        uidCell.setAttribute("label", EnigGetString("keylist.hasOtherUids"));
+        uidChildren.appendChild(subItem);
+        uidItem.setAttribute("open", "true");
+      }
+    }
+    if (treeItem)
+      treeChildren.appendChild(treeItem);
+
+  }
+
   // select last selected key
   if (selectedItems.length>0) {
     gUserList.view.selection.select(selectedItems[0]);
@@ -152,7 +244,7 @@ function enigmailBuildList(refresh) {
       gUserList.view.selection.rangedSelect(selectedItems[i], selectedItems[i], true)
     }
   }
-  gUserList.focus();
+  // gUserList.focus();
 }
 
 
@@ -339,10 +431,10 @@ function enigmailKeyMenu() {
     document.getElementById("bcEnableKey").removeAttribute("disabled");
     if (gKeyList[keyList[0]].keyUseFor.indexOf("D")>0 ||
         gKeyList[keyList[0]].keyTrust.indexOf(KEY_DISABLED)>=0) {
-      document.getElementById("bcEnableKey").setAttribute("label", EnigGetString("keyMan.enableKey"))
+      document.getElementById("bcEnableKey").setAttribute("label", EnigGetString("keyMan.enableKey"));
     }
     else {
-      document.getElementById("bcEnableKey").setAttribute("label", EnigGetString("keyMan.disableKey"))
+      document.getElementById("bcEnableKey").setAttribute("label", EnigGetString("keyMan.disableKey"));
     }
   }
 
@@ -368,6 +460,7 @@ function enigmailKeyMenu() {
   }
 }
 
+
 function enigmailOnClick(event) {
   if (event.detail != 2) {
     return;
@@ -407,10 +500,15 @@ function enigmailKeyDetails() {
 
   var inputObj = {
     keyId:  keyList[0],
+    keyListArr: gKeyList,
     secKey: gKeyList[ keyList[0]].secretAvailable
   };
+  var resultObj = { refresh: false };
   window.openDialog("chrome://enigmail/content/enigmailKeyDetailsDlg.xul",
-        "", "dialog,modal,centerscreen", inputObj);
+        "", "dialog,modal,centerscreen,resizable", inputObj, resultObj);
+  if (resultObj.refresh) {
+    enigmailRefreshKeys();
+  }
 }
 
 
@@ -426,11 +524,11 @@ function enigmailDeleteKey() {
     // one key selected
     var userId="0x"+keyList[0].substr(-8,8)+" - "+gKeyList[keyList[0]].userId;
     if(gKeyList[keyList[0]].secretAvailable) {
-      if (!EnigConfirm(EnigGetString("deleteSecretKey", userId))) return;
+      if (!EnigConfirm(EnigGetString("deleteSecretKey", userId), EnigGetString("dlg.button.delete"))) return;
       deleteSecret=true;
     }
     else {
-      if (!EnigConfirm(EnigGetString("deletePubKey", userId))) return;
+      if (!EnigConfirm(EnigGetString("deletePubKey", userId), EnigGetString("dlg.button.delete"))) return;
     }
   }
   else {
@@ -440,10 +538,10 @@ function enigmailDeleteKey() {
     }
 
     if (deleteSecret) {
-      if (!EnigConfirm(EnigGetString("deleteMix"))) return;
+      if (!EnigConfirm(EnigGetString("deleteMix"), EnigGetString("dlg.button.delete"))) return;
     }
     else {
-      if (!EnigConfirm(EnigGetString("deleteSelectedPubKey"))) return;
+      if (!EnigConfirmSring(EnigGetString("deleteSelectedPubKey"), EnigGetString("dlg.button.delete"))) return;
     }
   }
 
@@ -510,7 +608,7 @@ function enigCreateKeyMsg() {
   // save file
   var exitCodeObj= {};
   var errorMsgObj = {};
-  enigmailSvc.extractKey(window, 0, "0x"+keyList.join(" 0x"), tmpFile.path, exitCodeObj, errorMsgObj);
+  enigmailSvc.extractKey(window, 0, "0x"+keyList.join(" 0x"), tmpFile, exitCodeObj, errorMsgObj);
   if (exitCodeObj.value != 0) {
     EnigAlert(errorMsgObj.value);
     return;
@@ -538,24 +636,13 @@ function enigCreateKeyMsg() {
 
   var msgCompSvc = Components.classes["@mozilla.org/messengercompose;1"].getService(Components.interfaces.nsIMsgComposeService);
 
-  if (typeof(msgCompSvc.OpenComposeWindowWithCompFields) != "function") {
-    // TB 1.5
-    var msgCompParam = Components.classes["@mozilla.org/messengercompose/composeparams;1"].createInstance(Components.interfaces.nsIMsgComposeParams);
-    msgCompParam.composeFields = msgCompFields;
-    msgCompParam.identity = acctManager.defaultAccount.defaultIdentity;
-    msgCompParam.type = Components.interfaces.nsIMsgCompType.New;
-    msgCompParam.format = Components.interfaces.nsIMsgCompFormat.Default;
-    msgCompParam.originalMsgURI = "";
-    msgCompSvc.OpenComposeWindowWithParams("", msgCompParam);
-  }
-  else {
-    // TB 1.0
-    msgCompSvc.OpenComposeWindowWithCompFields ("",
-            Components.interfaces.nsIMsgCompType.New,
-            Components.interfaces.nsIMsgCompFormat.Default,
-            msgCompFields,
-            acctManager.defaultAccount.defaultIdentity);
-  }
+  var msgCompParam = Components.classes["@mozilla.org/messengercompose/composeparams;1"].createInstance(Components.interfaces.nsIMsgComposeParams);
+  msgCompParam.composeFields = msgCompFields;
+  msgCompParam.identity = acctManager.defaultAccount.defaultIdentity;
+  msgCompParam.type = Components.interfaces.nsIMsgCompType.New;
+  msgCompParam.format = Components.interfaces.nsIMsgCompFormat.Default;
+  msgCompParam.originalMsgURI = "";
+  msgCompSvc.OpenComposeWindowWithParams("", msgCompParam);
 }
 
 
@@ -571,8 +658,9 @@ function enigEditKeyTrust() {
     userIdList.push(gKeyList[keyList[i]].userId);
   }
 
-  EnigEditKeyTrust(userIdList, keyList);
-  enigmailRefreshKeys();
+  if (EnigEditKeyTrust(userIdList, keyList)) {
+    enigmailRefreshKeys();
+  }
 }
 
 
@@ -582,50 +670,16 @@ function enigSignKey() {
     EnigAlert(EnigGetString("noKeySelected"));
     return;
   }
-  EnigSignKey(gKeyList[keyList[0]].userId, keyList[0], null);
-  enigmailRefreshKeys();
+  if (EnigSignKey(gKeyList[keyList[0]].userId, keyList[0], null)) {
+    enigmailRefreshKeys();
+  }
 }
 
 function enigmailRevokeKey() {
   var keyList = enigmailGetSelectedKeys();
-
-  var enigmailSvc = GetEnigmailSvc();
-  if (!enigmailSvc)
-    return;
-
-  var userId="0x"+keyList[0].substr(-8,8)+" - "+gKeyList[keyList[0]].userId;
-  if (!EnigConfirm(EnigGetString("revokeKeyAsk", userId))) return;
-
-  var tmpDir=EnigGetTempDir();
-
-  try {
-    var revFile = Components.classes[ENIG_LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);
-    revFile.initWithPath(tmpDir);
-    if (!(revFile.isDirectory() && revFile.isWritable())) {
-      EnigAlert(EnigGetString("noTempDir"));
-      return;
-    }
+  if (EnigRevokeKey(keyList[0], gKeyList[keyList[0]].userId)) {
+    enigmailRefreshKeys();
   }
-  catch (ex) {}
-  revFile.append("revkey.asc");
-  revFile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
-
-  var errorMsgObj = {};
-  var r=enigmailSvc.genRevokeCert(window, "0x"+keyList[0], revFile.path, "0", "", errorMsgObj);
-  if (r != 0) {
-    revFile.remove(false);
-    EnigAlert(EnigGetString("revokeKeyFailed")+"\n\n"+errorMsgObj.value);
-    return;
-  }
-  r = enigmailSvc.importKeyFromFile(window, revFile.path, errorMsgObj);
-  revFile.remove(false);
-  if (r != 0) {
-    EnigAlert(EnigGetString("revokeKeyFailed")+"\n\n"+EnigConvertGpgToUnicode(errorMsgObj.value).replace(/\\e3A/g, ":"));
-  }
-  else {
-    EnigAlert(EnigGetString("revokeKeyOk"));
-  }
-  enigmailRefreshKeys();
 }
 
 function enigCreateRevokeCert() {
@@ -643,7 +697,7 @@ function enigmailExportKeys() {
 
   var exportFlags = 0;
   if (gKeyList[keyList[0]].secretAvailable) {
-    if (EnigConfirm(EnigGetString("exportSecretKey"))) {
+    if (EnigConfirm(EnigGetString("exportSecretKey"), EnigGetString("keyMan.button.exportSecKey"))) {
       exportFlags |= nsIEnigmail.EXTRACT_SECRET_KEY;
     }
   }
@@ -681,7 +735,7 @@ function enigmailExportKeys() {
   var keyListStr = "0x"+keyList.join(" 0x");
   var exitCodeObj = {};
   var errorMsgObj = {};
-  enigmailSvc.extractKey(window, exportFlags, keyListStr, outFile.path, exitCodeObj, errorMsgObj);
+  enigmailSvc.extractKey(window, exportFlags, keyListStr, outFile, exitCodeObj, errorMsgObj);
   if (exitCodeObj.value != 0) {
     EnigAlert(EnigGetString("saveKeysFailed")+"\n\n"+errorMsgObj.value);
   }
@@ -702,7 +756,7 @@ function enigmailImportKeysFromFile() {
   if (! inFile) return;
 
   var errorMsgObj = {};
-  var exitCode =enigmailSvc.importKeyFromFile(window, inFile.path, errorMsgObj);
+  var exitCode =enigmailSvc.importKeyFromFile(window, inFile.persistentDescriptor, errorMsgObj);
   if (exitCode != 0) {
     EnigAlert(EnigGetString("importKeysFailed")+"\n\n"+errorMsgObj.value);
   }
@@ -751,38 +805,17 @@ function enigmailManageUids() {
     keyId: keyList[0],
     ownKey: gKeyList[keyList[0]].secretAvailable
   };
-
+  var resultObj = { refresh: false };
   window.openDialog("chrome://enigmail/content/enigmailManageUidDlg.xul",
-        "", "dialog,modal,centerscreen,resizable=yes", inputObj);
-  enigmailRefreshKeys();
+        "", "dialog,modal,centerscreen,resizable=yes", inputObj, resultObj);
+  if (resultObj.refresh) {
+    enigmailRefreshKeys();
+  }
 }
 
 function enigmailChangePwd() {
   var keyList = enigmailGetSelectedKeys();
-  var inputObj = {
-    keyId: keyList[0],
-    userId: gKeyList[keyList[0]].userId
-  };
-
-  var enigmailSvc = GetEnigmailSvc();
-  if (!enigmailSvc)
-    return;
-
-  if (! enigmailSvc.useGpgAgent()) {
-    window.openDialog("chrome://enigmail/content/enigmailChangePasswd.xul",
-        "", "dialog,modal,centerscreen", inputObj);
-  }
-  else {
-    // gpg-agent will handle everything
-    var errorMsgObj = new Object();
-    var r = enigmailSvc.simpleChangePassphrase(window, keyList[0], errorMsgObj);
-
-    if (r != 0) {
-      EnigAlert(EnigGetString("changePassFailed")+"\n\n"+errorMsgObj.value);
-
-    }
-  }
-
+  EnigChangeKeyPwd(keyList[0], gKeyList[keyList[0]].userId);
 }
 
 
@@ -810,7 +843,7 @@ function enigmailImportFromClipbrd() {
   if (!enigmailSvc)
     return;
 
-  if (!EnigConfirm(EnigGetString("importFromClip"))) {
+  if (!EnigConfirm(EnigGetString("importFromClip"), EnigGetString("keyMan.button.import"))) {
     return;
   }
 
@@ -889,9 +922,20 @@ function enigmailReceiveKey() {
 }
 
 function enigmailRefreshAllKeys() {
-
-  EnigAlertPref(EnigGetString("refreshKey.warn"), "warnRefreshAll");
-  enigmailKeyServerAcess(nsIEnigmail.REFRESH_KEY, enigmailReceiveKeyCb);
+  var checkedObj = {};
+  var doIt=false;
+  if (!EnigGetPref("warnRefreshAll")) {
+    doIt=true;
+  }
+  else if (EnigLongAlert(EnigGetString("refreshKey.warn"), EnigGetString("dlgNoPrompt"), 
+      EnigGetString("dlg.button.continue"), ":cancel", null, checkedObj) == 0) {
+      if (checkedObj.value) {
+        EnigSetPref("warnRefreshAll", false)
+      }
+    doIt=true;
+  }
+  
+  if (doIt) enigmailKeyServerAcess(nsIEnigmail.REFRESH_KEY, enigmailReceiveKeyCb);
 }
 
 function enigmailReceiveKeyCb(exitCode, errorMsg, msgBox) {
@@ -927,7 +971,7 @@ function onSearchInput(returnKeyHit)
     onEnterInSearchBar();
   }
   else {
-    gSearchTimer = setTimeout("onEnterInSearchBar();", 800);
+    gSearchTimer = setTimeout("onEnterInSearchBar();", 600);
   }
 }
 
@@ -951,23 +995,68 @@ function onEnterInSearchBar() {
 }
 
 function getFirstNode() {
-  return document.getElementById("pgpKeyListChildren").firstChild;
+  return gTreeChildren.firstChild;
 }
 
 function onResetFilter() {
   gFilterBox.value="";
+  showOrHideAllKeys();
+  /*
+  if (! displayFullList()) {
+    showOrHideAllKeys();
+  }
+  else {
+    var node=getFirstNode();
+    while (node) {
+      node.hidden=false;
+      node = node.nextSibling;
+    }
+  } */
+  gClearButton.setAttribute("disabled", true);
+}
+
+function enigmailToggleShowAll() {
+  // gShowAllKeysElement.checked = (! gShowAllKeysElement.checked);
+  EnigSetPref("keyManShowAllKeys", displayFullList());
+
+  if (!gSearchInput.value || gSearchInput.value.length==0) {
+    showOrHideAllKeys();
+  }
+}
+
+
+function showOrHideAllKeys() {
+  var hideNode = ! displayFullList();
+  var initHint = document.getElementById("emptyTree");
+  document.getElementById("nothingFound").hidePopup();
+  if (hideNode) {
+    initHint.showPopup(gTreeChildren, -1, -1, "tooltip", "after_end", "");
+  }
+  else {
+    initHint.hidePopup();
+  }
   var node=getFirstNode();
   while (node) {
-    node.hidden=false;
+    node.hidden= hideNode;
     node = node.nextSibling;
   }
-  gClearButton.setAttribute("disabled", true);
 }
 
 function enigApplyFilter() {
   var searchTxt=gSearchInput.value;
-  if (!searchTxt || searchTxt.length==0) return;
+  var nothingFoundElem = document.getElementById("nothingFound");
+  nothingFoundElem.hidePopup();
+
+  if (!searchTxt || searchTxt.length==0) {
+    showOrHideAllKeys();
+    return;
+  }
+  else {
+    document.getElementById("emptyTree").hidePopup();
+  }
+
   searchTxt = searchTxt.toLowerCase();
+  var foundResult = false;
   var node=getFirstNode();
   while (node) {
     var uid = gKeyList[node.id].userId;
@@ -975,16 +1064,22 @@ function enigApplyFilter() {
     if ((uid.toLowerCase().indexOf(searchTxt) >= 0) ||
         (node.id.toLowerCase().indexOf(searchTxt) >= 0)) {
       hideNode = false;
+      foundResult = true;
     }
     for (var subUid=0; subUid < gKeyList[node.id].SubUserIds.length; subUid++) {
       uid = gKeyList[node.id].SubUserIds[subUid].userId;
       if (uid.toLowerCase().indexOf(searchTxt) >= 0) {
         hideNode = false;
+        foundResult = true;
       }
     }
     node.hidden=hideNode;
     node = node.nextSibling;
   }
+
+  if (! foundResult) {
+    nothingFoundElem.showPopup(gTreeChildren, -1, -1, "tooltip", "after_end", "");
+  }
 }
 
 //
@@ -1004,7 +1099,7 @@ function enigmailKeyServerAcess(accessType, callbackFunc) {
 
   var selKeyList = enigmailGetSelectedKeys();
   if (accessType != nsIEnigmail.REFRESH_KEY && selKeyList.length==0) {
-    if (EnigConfirm(EnigGetString("refreshAllQuestion"))) {
+    if (EnigConfirm(EnigGetString("refreshAllQuestion"), EnigGetString("keyMan.button.refreshAll"))) {
       accessType = nsIEnigmail.REFRESH_KEY;
       EnigAlertPref(EnigGetString("refreshKey.warn"), "warnRefreshAll");
     }
