@@ -33,6 +33,7 @@
 
 var gEnigModifySettings;
 var gLastDirection=0;
+var gEnigAccountMgr;
 
 EnigInitCommon("enigmailSetupWizard");
 
@@ -44,13 +45,15 @@ function onLoad() {
     quotedPrintable: true,
     composeHTML: true
   };
+  gEnigAccountMgr = Components.classes["@mozilla.org/messenger/account-manager;1"].getService(Components.interfaces.nsIMsgAccountManager);
+
   fillIdentities('checkbox');
 }
 
 
 function onCancel() {
   if (gKeygenRequest) {
-    if (EnigConfirm(EnigGetString("keyAbort"))) {
+    if (EnigConfirm(EnigGetString("keyAbort"), EnigGetString("keyMan.button.generateKeyAbort"), EnigGetString("keyMan.button.generateKeyContinue"))) {
       enigmailKeygenCloseRequest();
       return true;
     }
@@ -59,7 +62,7 @@ function onCancel() {
     }
   }
   else {
-    return EnigConfirm(EnigGetString("setupWizard.reallyCancel"))
+    return (EnigLongAlert(EnigGetString("setupWizard.reallyCancel"), null, EnigGetString("dlg.button.close"), EnigGetString("dlg.button.continue")) == 0);
   }
 }
 
@@ -133,10 +136,10 @@ function displayKeyCreate() {
     var node = document.getElementById("idSelection").firstChild;
     while (node) {
       if (node.checked) {
-        var identity = gAccountManager.getIdentity(node.getAttribute("account-id"));
+        var identity = gEnigAccountMgr.getIdentity(node.getAttribute("account-id"));
         var idName = identity.identityName;
 
-        var serverSupports = gAccountManager.GetServersForIdentity(identity);
+        var serverSupports = gEnigAccountMgr.GetServersForIdentity(identity);
         if (serverSupports.GetElementAt(0)) {
           var inServer = serverSupports.GetElementAt(0).QueryInterface(Components.interfaces.nsIMsgIncomingServer);
 
@@ -207,61 +210,30 @@ function loadKeys() {
   var exitCodeObj = {};
   var statusFlagsObj = {};
   var errorMsgObj = {};
-  var keyList = enigmailSvc.getUserIdList(true, true, exitCodeObj, statusFlagsObj, errorMsgObj);
-  if (exitCodeObj.value != 0) {
-    EnigAlert(errorMsgObj.value);
+  var keyList = EnigGetSecretKeys();
+  if (keyList == null) {
     return false;
   }
 
-  if (keyList.search(/^sec:/m) < 0) {
+  if (keyList.length ==0) {
     setNextPage("pgKeyCreate");
     return true;
   }
 
-  var uidList = [];
-  var uidObj = {};
-  var keyArr = keyList.split(/[\r\n]/);
-  var now = Number(new Date())/1000;
-  for (var i=0; i<keyArr.length; i++) {
-    var l = keyArr[i].split(/:/);
-    switch(l[0]) {
-    case "sec":
-      if ((l[6]=="") || (l[6] > now)) {
-        uidObj = {
-          keyId: l[4],
-          uid: "",
-          created: EnigGetDateTime(l[5], true, false)
-        };
-        uidList.push(uidObj);
-      }
-      else {
-        uidObj = {
-          keyId: "-",
-          uid: ""
-        }
-      }
-      break;
-    case "uid":
-      if ((uidObj.keyId !="-") && (uidObj.uid == "")) {
-        uidObj.uid = EnigConvertGpgToUnicode(l[9]).replace(/\\e3A/g, ":");
-      }
-      break;
-    }
-  }
 
   var uidChildren = document.getElementById("uidSelectionChildren")
-  for (i=0; i<uidList.length; i++) {
+  for (i=0; i < keyList.length; i++) {
     var item = uidChildren.appendChild( document.createElement('treeitem') );
-    item.setAttribute("keyId", uidList[i].keyId);
+    item.setAttribute("keyId", keyList[i].id);
     var row = item.appendChild(document.createElement('treerow'));
     var cell = row.appendChild( document.createElement('treecell') )
-    cell.setAttribute('label', uidList[i].uid);
+    cell.setAttribute('label', keyList[i].name);
     cell.setAttribute('observes', "bcKeyEnabled");
     cell = row.appendChild( document.createElement('treecell') );
-    cell.setAttribute('label', "0x"+uidList[i].keyId.substr(-8,8));
+    cell.setAttribute('label', "0x"+keyList[i].id.substr(-8,8));
     cell.setAttribute('observes', "bcKeyEnabled");
     cell = row.appendChild( document.createElement('treecell') );
-    cell.setAttribute('label', uidList[i].created);
+    cell.setAttribute('label', keyList[i].created);
     cell.setAttribute('observes', "bcKeyEnabled");
   }
   onKeySelected();
@@ -310,7 +282,7 @@ function enigGetSvc() {
         }
       }
       else {
-        EnigSetPref("agentPath", gpgPath.path);
+        EnigSetPref("agentPath", gpgPath.persistentDescriptor);
       }
     }
 
@@ -374,7 +346,7 @@ function wizardGenKey() {
                                          userEmail,
                                          365*5 /* 5 years */,
                                          2048,
-                                         ENIG_KEYTYPE_DSA,
+                                         ENIG_KEYTYPE_RSA,
                                          passphrase,
                                          requestObserver);
   } catch (ex) {}
@@ -445,7 +417,7 @@ function fillIdentities(fillType)
 
     // Find out default identity
     var defIdentity;
-    var defIdentities = gAccountManager.defaultAccount.identities;
+    var defIdentities = gEnigAccountMgr.defaultAccount.identities;
     if (defIdentities.Count() >= 1) {
       defIdentity = defIdentities.QueryElementAt(0, Components.interfaces.nsIMsgIdentity);
     } else {
@@ -457,7 +429,7 @@ function fillIdentities(fillType)
       var node = document.getElementById("idSelection").firstChild;
       while (node) {
         if (node.checked) {
-          var currId = gAccountManager.getIdentity(node.getAttribute("account-id"));
+          var currId = gEnigAccountMgr.getIdentity(node.getAttribute("account-id"));
           if (currId.key == defIdentity.key) {
             break;
           }
@@ -470,7 +442,7 @@ function fillIdentities(fillType)
         node = document.getElementById("idSelection").firstChild;
         while (node) {
           if (node.checked) {
-            defIdentity = gAccountManager.getIdentity(node.getAttribute("account-id"));
+            defIdentity = gEnigAccountMgr.getIdentity(node.getAttribute("account-id"));
             break;
           }
           node = node.nextSibling;
@@ -484,7 +456,7 @@ function fillIdentities(fillType)
     parentElement.removeChild(child);
     child=parentElement.firstChild;
   }
-  var idSupports = gAccountManager.allIdentities;
+  var idSupports = gEnigAccountMgr.allIdentities;
   var identities = queryISupportsArray(idSupports,
                                        Components.interfaces.nsIMsgIdentity);
 
@@ -499,7 +471,7 @@ function fillIdentities(fillType)
     if (!identity.valid || !identity.email)
       continue;
 
-    var serverSupports = gAccountManager.GetServersForIdentity(identity);
+    var serverSupports = gEnigAccountMgr.GetServersForIdentity(identity);
 
     if (serverSupports.GetElementAt(0)) {
       var inServer = serverSupports.GetElementAt(0).QueryInterface(Components.interfaces.nsIMsgIncomingServer);
@@ -547,7 +519,7 @@ function wizardGetSelectedIdentity()
   var item = document.getElementById("userIdentity").selectedItem;
   var identityKey = item.getAttribute('account-id');
 
-  return gAccountManager.getIdentity(identityKey);
+  return gEnigAccountMgr.getIdentity(identityKey);
 }
 
 function applyWizardSettings() {
@@ -556,7 +528,7 @@ function applyWizardSettings() {
   loadLastPage();
 
   if (document.getElementById("activateId").value == "1") {
-    var idSupports = gAccountManager.allIdentities;
+    var idSupports = gEnigAccountMgr.allIdentities;
     var identities = queryISupportsArray(idSupports,
                                        Components.interfaces.nsIMsgIdentity);
     for (var i=0; i<identities.length; i++) {
@@ -567,7 +539,7 @@ function applyWizardSettings() {
     var node = document.getElementById("idSelection").firstChild;
     while (node) {
       if (node.checked) {
-        var identity = gAccountManager.getIdentity(node.getAttribute("account-id"));
+        var identity = gEnigAccountMgr.getIdentity(node.getAttribute("account-id"));
         wizardApplyId(identity, gGeneratedKey);
       }
       node = node.nextSibling;
@@ -659,7 +631,7 @@ function wizardKeygenTerminate(terminateArg, ipcRequest) {
   ipcRequest.close(true);
   var curId = wizardGetSelectedIdentity();
 
-  if (EnigConfirm(EnigGetString("keygenComplete", curId.email)+"\n\n"+EnigGetString("revokeCertRecommended"))) {
+  if (EnigConfirm(EnigGetString("keygenComplete", curId.email)+"\n\n"+EnigGetString("revokeCertRecommended"), EnigGetString("keyMan.button.generateCert"), EnigGetString("dlg.button.skip"))) {
     EnigCreateRevokeCert(gGeneratedKey, curId.email);
   }
 
@@ -784,7 +756,7 @@ function displayActions() {
       var node = document.getElementById("idSelection").firstChild;
       while (node) {
         if (node.checked) {
-          var identity = gAccountManager.getIdentity(node.getAttribute("account-id"));
+          var identity = gEnigAccountMgr.getIdentity(node.getAttribute("account-id"));
           idList+="<"+identity.email+"> "
         }
         node = node.nextSibling;
